% Fleur ter Huurne  | 4 august 2015
% NDJF circuit - fig 7A Boareto - 20 interacting cells
% Edited by Sandra Loerakker, 5 July 2016
%   - Added comments
% Edited by Sandra Loerakker, 8 September 2016
%   - Added comments
% Edited by Sandra Loerakker, 9 December 2016
%   - Removed separation of Ne (increase similarity with equations in paper)
% Edited by Jonne de Bijl
%works for wksp_N1800_changingN_nopattern
% Edited by Tommaso Ristori, 25/10/2018
% inclusion of the second model of Boareto with Vegf and Vegfr
% Edited by Tommaso Ristori 11/28/2020
% inclusion of Jag1 pattern, homogeneous and not homogeneous, and make a
% function out of it

function EfficiencyCoeff = Boareto_next_DeltaJaggedPattern_Cluster(ncells_range, Dp, Jp, patternyesno, sFileName)

%clearvars
close all
%clc

% ------------------------------------------------------------------------
% constants
gamma = 0.1;    % [1/h] degradation of Notch (N), Delta (D), Jagged (J)
gammaI = 0.5;   % [1/h] degradation of NICD (I)
kc = 5e-04;     % [1/h] strength cis-inhibition
kt = 2.5e-05;   % [1/h] strengtth transactivation
Npr = 1200;     % [number of proteins] production rate Notch in molec
Dpr = 1000;      % [number of proteins] production rate Delta in molec 
Jpr = 800;     % [number of proteins] production rate Delta in molec % 1200 for high values
I0 = 200;       % [number of proteins] threshold of the Hill function in molec
lN = 2.0;       % Lambda Notch of shifted Hill function [dimensionless]
lD = 0.0;       % Lambda Delta of shifted Hill function [dimensionless]
lJ = 2.0;       % Lambda Jagged of shifted Hill function [dimensionless]
nN = 2.0;       % Hill Notch coefficients [dimensionless]
nD = 2.0;       % Hill Delta coefficients [dimensionless]
nJ = 5.0;       % Hill Jagged coefficients [dimensionless]

% parameters for the Fringe
nf = 1.0;       % Hill Fringe coefficient [dimensionless]
lDf = 3.0;      % Lambda Delta of shifted Hill function [dimensionless]
lJf = 0.3;      % Lambda Jagged of shifted Hill function [dimensionless]

% parameters for the vegf and vegfr
nV = 2.0; % Hill vegf-Dll4 coeff (increase of Dll4 when vegf is received) [dimensionless] 
lV = 2.0; % Lambda of vegf-Delta shifted Hill function (increase of Dll4 when vegf is received) [dimensionless]
VRpr = 1000; % [number of proteins/h] production rate of Vegfr in molec
nVR = 2.0; % Hill vegfr-vegf coeff (increase of Vegf sensed by the cell)
lVR = 0.0;
V0 = 200;       % [number of proteins] threshold of the Hill function of vegf in molec
Vext = 1000; % [number of proteins] concentration of vegf in the surrounding

% %values of Dp and Jp on the patterns
% Dp= 2250; %average based on Neighbors average divided by two (other side of cell no interaction)
% Jp = 0;
% % do you want with patterns or homogeneous?
% patternyesno = 1; % 1 yes pattern; 0 no pattern, homogeneous coating

% -------------------------------------------------------------------------
% variables 
tmin = 0;  % start time
tmax = 12; % end time - 6 hours for homogeneous coating experiments and 12h for angiogenesis
dt = 0.01; % time step (dt)
ncells = 5; % number of cells 
ncombs = 10000; % number of simulations - take average over 100 simulations

% -------------------------------------------------------------------------
%ncells_range=12;

N_total = zeros(length(ncells_range),ncells_range(end));
D_total = zeros(length(ncells_range),ncells_range(end));
J_total = zeros(length(ncells_range),ncells_range(end));
I_total = zeros(length(ncells_range),ncells_range(end));
VR_total = zeros(length(ncells_range),ncells_range(end));
V_total = zeros(length(ncells_range),ncells_range(end));

% Save number of states per length (multiple simulations)
nS = zeros(ncombs,1);
nSR = zeros(ncombs,1);
nR = zeros(ncombs,1);

% Save number of cell states and average protein levels for different lengths
ncellstates1 = zeros(length(ncells_range),4);
proteins_levels1 = zeros(length(ncells_range),8);

% Save average protein levels per cell for different lengths
N_mean = zeros(length(ncells_range),ncells_range(end));
D_mean = zeros(length(ncells_range),ncells_range(end));
J_mean = zeros(length(ncells_range),ncells_range(end));
I_mean = zeros(length(ncells_range),ncells_range(end));
I_end = zeros(length(ncells_range),ncells_range(end));
VR_mean = zeros(length(ncells_range),ncells_range(end));
VR_end = zeros(length(ncells_range),ncells_range(end));
V_mean = zeros(length(ncells_range),ncells_range(end));
V_end = zeros(length(ncells_range),ncells_range(end));

for ppp = 1:length(ncells_range)
    ncells = ncells_range(ppp)
    if mod(ncells,4) > 0
        error('insert ncells multiple of 4')
    end
    
    t = zeros((tmax/dt),1);
    t(1) = tmin;
    
    % array with protein numbers in each cell at each time point 
    Notch = zeros((tmax/dt),ncells);
    Delta = zeros((tmax/dt),ncells);
    Jagged = zeros((tmax/dt),ncells);
    NICD = zeros((tmax/dt),ncells);
    VR = zeros((tmax/dt),ncells);
    V = zeros((tmax/dt),ncells);
    
    % array with protein numbers in each cell at t = tmax for all
    N_combs = zeros(ncombs,ncells);
    D_combs = zeros(ncombs,ncells);
    J_combs = zeros(ncombs,ncells);
    I_combs = zeros(ncombs,ncells);
    VR_combs = zeros(ncombs,ncells);
    V_combs = zeros(ncombs,ncells);
    N_expcombs = zeros(ncombs,ncells);
    D_expcombs = zeros(ncombs,ncells);
    J_expcombs = zeros(ncombs,ncells);
    VR_expcombs = zeros(ncombs,ncells);
        
    for s = 1:ncombs
        % random initial condition for each cell, chosen between a certain
        % range (based on bifurgation plots Boareto)
        % same ranges as Boareto used were chosen
        rng('shuffle') % still random after restarting Matlab
        for c_int = 1:ncells
            % value for N, D, J and I at tmin (initial value)
            Notch(1,c_int) = randi([0, 6000],1,1);
            Delta(1,c_int) = randi([0, 6000],1,1);
            Jagged(1,c_int) = randi([0, 6000],1,1);
            NICD(1,c_int) = randi([0, 600],1,1);
            VR(1,c_int) = randi([0, 6000],1,1);
            V(1,c_int) = randi([0, 600],1,1);
        end
        
        i = 0;
        for time = tmin+dt:dt:tmax
            i = i + 1;
            % calculate Ne, De and Je for each cell
            % function Hillfunction = HS(I,I0,n,l)
            % function Neighbors to compute the Notch proteins in the neighborhood
            Neighbors = extp_F_dt_oneway_DeltaJaggedPattern...
                (ncells,Notch,Delta,Jagged,NICD,i,Dp,Jp,patternyesno);
            NeD = Neighbors(1,:); % number of Notch molecules in neighboring cells
            NeJ = Neighbors(2,:); % number of Notch molecules in neighboring cells
            De = Neighbors(3,:); % number of Delta molecules in neighboring cells 
            Je = Neighbors(4,:); % number of Jagged molecules in neighboring cells
            
            t(i+1) = time;
            % calculate number of proteins (N,D,J,I) at each time point
            % expicit solution t_new = t_old + differential equations
            % store number of proteins at the next time step in the next row
            Notch(i+1,:) = Notch(i,:) + dt*(Npr*HS(NICD(i,:),I0,nN,lN) - Notch(i,:).*((kc*Delta(i,:) + kt*De) ...
                .*HS(NICD(i,:),I0,nf,lDf) + (kc*Jagged(i,:) + kt*Je).*HS(NICD(i,:),I0,nf,lJf)) - gamma*Notch(i,:));
            Delta(i+1,:) = Delta(i,:) + dt*(Dpr*HS(NICD(i,:),I0,nD,lD).*HS(V(i,:),V0,nV,lV) - Delta(i,:).*(kc*Notch(i,:) + kt*NeD) ...
                .*HS(NICD(i,:),I0,nf,lDf) - gamma*Delta(i,:));
            Jagged(i+1,:) = Jagged(i,:) + dt*(Jpr*HS(NICD(i,:),I0,nJ,lJ) - Jagged(i,:).*(kc*Notch(i,:) + kt*NeJ) ...
                .*HS(NICD(i,:),I0,nf,lJf) - gamma*Jagged(i,:));
            NICD(i+1,:) = NICD(i,:) + dt*(kt*Notch(i,:).*(De.*HS(NICD(i,:),I0,nf,lDf) + Je.*HS(NICD(i,:),I0,nf,lJf)) ...
                - gammaI.*NICD(i,:));
            VR(i+1,:) = VR(i,:) + dt*(VRpr*HS(NICD(i,:),V0,nVR,lVR)-kt*VR(i,:)*Vext-gamma*VR(i,:));
            V(i+1,:) = V(i,:) + dt*(kt*VR(i,:)*Vext - gammaI*V(i,:));
            
            for r = 1:ncells
                if Notch(i+1,r) < 0
                    Notch(i+1,r) = 0;
                end
                if Delta(i+1,r) < 0
                    Delta(i+1,r) = 0;
                end
                if Jagged(i+1,r) < 0
                    Jagged(i+1,r) = 0;
                end
                if NICD(i+1,r) < 0
                    NICD(i+1,r) = 0;
                end
                if VR(i+1,r) < 0
                    VR(i+1,r) = 0;
                end
                if V(i+1,r) < 0
                    V(i+1,r) = 0;
                end
            end
        end
        
        % save for each simulation the value of N, D, J and I at time = tmax
        % s is simulation number (s/ncombs)
        N_combs(s,:) = Notch(end,:);
        D_combs(s,:) = Delta(end,:);
        J_combs(s,:) = Jagged(end,:);
        I_combs(s,:) = NICD(end,:);
        VR_combs(s,:) = VR(end,:);
        V_combs(s,:) = V(end,:);
        N_expcombs(s,:) = Npr*HS(NICD(end,:),I0,nN,lN);
        D_expcombs(s,:) = Dpr*HS(NICD(end,:),I0,nD,lD).*HS(V(end,:),V0,nV,lV);
        J_expcombs(s,:) = Jpr*HS(NICD(end,:),I0,nJ,lJ);
        VR_expcombs(s,:) = VRpr*HS(NICD(end,:),V0,nVR,lVR);
        
    end % end for s = 1:ncombs
    
    % store average protein levels per cell for each length
    N_mean(ppp,1:ncells) = mean(N_combs,1);
    D_mean(ppp,1:ncells) = mean(D_combs,1);
    J_mean(ppp,1:ncells) = mean(J_combs,1);
    I_mean(ppp,1:ncells) = mean(I_combs,1);
    I_end(ppp,1:ncells) = I_combs(end,:);
    VR_mean(ppp,1:ncells) = mean(VR_combs,1);
    VR_end(ppp,1:ncells) = VR_combs(end,:);
    V_mean(ppp,1:ncells) = mean(V_combs,1);
    V_end(ppp,1:ncells) = V_combs(end,:);
    
    %store average protein expression per cell for each length
    N_expmean(ppp,1:ncells) = mean(N_expcombs,1);
    D_expmean(ppp,1:ncells) = mean(D_expcombs,1);
    J_expmean(ppp,1:ncells) = mean(J_expcombs,1);
    VR_expmean(ppp,1:ncells) = mean(VR_expcombs,1);
    
    % STD?? N_std(ppp,1:cells) = std(N_combs,1)
    N_std(ppp,1:ncells) = std(N_combs);
    D_std(ppp,1:ncells) = std(D_combs);
    J_std(ppp,1:ncells) = std(J_combs);
    I_std(ppp,1:ncells) = std(I_combs);
    VR_std(ppp,1:ncells) = std(VR_combs);
    V_std(ppp,1:ncells) = std(V_combs);
    
    % store average + standard deviation protein level per length
    proteins_levels1(ppp,1) = ncells;
    proteins_levels1(ppp,2) = mean(mean(N_combs));
    proteins_levels1(ppp,3) = mean(std(N_combs));
    proteins_levels1(ppp,4) = mean(mean(D_combs));
    proteins_levels1(ppp,5) = mean(std(D_combs));
    proteins_levels1(ppp,6) = mean(mean(J_combs));
    proteins_levels1(ppp,7) = mean(std(J_combs));
    proteins_levels1(ppp,8) = mean(mean(I_combs));
    proteins_levels1(ppp,9) = mean(std(I_combs));
    proteins_levels1(ppp,10) = mean(mean(VR_combs));
    proteins_levels1(ppp,11) = mean(std(VR_combs));
    proteins_levels1(ppp,12) = mean(mean(V_combs));
    proteins_levels1(ppp,13) = mean(std(V_combs));
    
    expression_levels1(ppp,1) = mean(mean(N_expcombs));
    expression_levels1(ppp,2) = std(mean(N_expcombs));
    expression_levels1(ppp,3) = mean(mean(D_expcombs));
    expression_levels1(ppp,4) = std(mean(D_expcombs));
    expression_levels1(ppp,5) = mean(mean(J_expcombs));
    expression_levels1(ppp,6) = std(mean(J_expcombs));
    expression_levels1(ppp,7) = mean(mean(VR_expcombs))
    expression_levels1(ppp,8) = std(mean(VR_expcombs));
    
    % determine cell fate based on V_combs value
    cstate = zeros(ncombs,ncells);
    for q = 1:size(V_combs,1)
        for w = 1:size(V_combs,2)
            if V_combs(q,w) < 100 
                cstate(q,w) = 0; %100 stalk cells
            elseif V_combs(q,w) > 300
                cstate(q,w) = 1; %300 tip cells
            else
                cstate(q,w)=0.5; %300 tip/stalk cells
            end
        end
    end
    
    % calculate number of differentiated and undifferentiated cells for each simulation
    for z = 1:ncombs 
        v = cstate(z,:);
        nT(z) = length(find(v == 1));
        nS(z) = length(find(v == 0));
    end
    
    % calculate the percentage of times a cell is tip cell based on
    % position
    for z = 1:size(cstate,2)
        v = cstate(:,z);
        percentageT(z) = length(find(v == 1))/size(cstate,1);
        percentageS(z) = length(find(v == 0))/size(cstate,1);
    end

    % calculate the Efficiency coefficient per repetition
    for z = 1:size(cstate,1)
        v = cstate(z,ncells/4+1:(ncells-ncells/4));
        TipOff(z) = length(find(v == 1))/size(cstate,2)+length(find(v == 0.5))/size(cstate,2);
        v = [cstate(z,1:ncells/4),cstate(z,(ncells-ncells/4+1:ncells))];
        TipOn(z) = length(find(v == 1))/size(cstate,2)+length(find(v == 0.5))/size(cstate,2);
        EfficiencyCoeff(z) = TipOff(z)/(TipOff(z)+TipOn(z));
    end
    
    % Calculate the average and std of the number of states for all simulations
    mean_nT = mean(nT);
    mean_nS = mean(nS);
    std_nT = std(nT);
    std_nS = std(nS);
    
    % mean values of ncombs
    ncellstates1(ppp,1) = ncells;
%     ncellstates1(ppp,2) = h;
%     ncellstates1(ppp,3) = T;
%     ncellstates1(ppp,4) = length(nS);
%     ncellstates1(ppp,5) = length(nSR);
%     ncellstates1(ppp,6) = length(nR);
%     ncellstates1(ppp,7) = length(nS)/ncells*100;
%     ncellstates1(ppp,8) = length(nSR)/ncells*100;
%     ncellstates1(ppp,9) = length(nR)/ncells*100;
    ncellstates1(ppp,4) = mean_nT;
    ncellstates1(ppp,6) = mean_nS;
    ncellstates1(ppp,7) = std_nT;
    ncellstates1(ppp,9) = std_nS;    
    ncellstates1(ppp,10) = mean_nT/ncells*100;
    ncellstates1(ppp,12) = mean_nS/ncells*100;
    ncellstates1(ppp,13) = std_nT/ncells*100;
    ncellstates1(ppp,15) = std_nS/ncells*100;
   
end % end for r = 1:length(Dpr_range)

save(sFileName);
end

